'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

const index = require('./index-ceff47a0.js');
const index_esm = require('./index.esm-beac1e98.js');
const index_esm$1 = require('./index.esm-ab93ebfa.js');
const index_esm$2 = require('./index.esm-7e3242f9.js');
const reduxStore_service = require('./redux-store.service-dfd8922f.js');

const bpSdkNextButtonContainerCss = "";
const BpSdkNextButtonContainerStyle0 = bpSdkNextButtonContainerCss;

const BpSdkNextButtonContainer = class {
    constructor(hostRef) {
        index.registerInstance(this, hostRef);
        this.bpAnalyticsTrack = index.createEvent(this, "bpAnalyticsTrack", 7);
        this.handleSubscriptions = () => {
            if (this.storeUnsubscribe) {
                this.storeUnsubscribe();
            }
            reduxStore_service.store.mapDispatchToProps(this, {
                saveQuestionSet: index_esm.Le,
                updateErrors: index_esm.wx,
                removeActiveAnswerId: index_esm.WC,
                switchActiveQuestionSet: index_esm.GC,
                pushAnswer: index_esm.St,
                getEligibilityResults: index_esm.Or,
                setAddressValidationStatus: index_esm.Rs,
                setAddressValidationTriggerOverridden: index_esm.Pv,
            });
            this.storeUnsubscribe = reduxStore_service.store.mapStateToProps(this, this.mapStateToProps);
        };
        this.mapStateToProps = (state) => ({
            activeQuestionSet: index_esm.T(state),
            applicationId: index_esm.Um(state),
            config: index_esm.mr(state),
            answersResourceState: index_esm.hg(state),
            applicationFormResourceState: index_esm.wg(state),
            pendingProcesses: state.application.pendingProcesses,
            questionPoolResourceState: index_esm.Pg(state),
            questionSetResourceState: index_esm.bg(state),
            questionSetState: index_esm.ne(state),
            queue: state.queues.answers.queue,
            enableUserApplicationGuard: !!state.config.options.enableUserApplicationGuard,
            locked: state.application.form.locked,
            isActiveQuestionSetPendingProcess: index_esm.vS(state),
            isOwner: index_esm.bS(state),
            hasPermissionToEdit: index_esm.ES(state, {
                permission: index_esm$1.H.ApplicationForms,
                action: index_esm$1.Y.Update,
            }),
            addressValidationStatuses: index_esm.LA(state),
            addressIdsToBeValidated: index_esm.OA(state),
            isValidationTriggerOverridden: index_esm.BA(state),
            isActiveQuestionSetBeingValidated: index_esm.TA(state, this.activeQuestionSet),
        });
        this.activeQuestionSet = undefined;
        this.buttonDisabled = false;
        this.config = undefined;
        this.queue = undefined;
        this.shouldDisableIfIncomplete = false;
        this.answersResourceState = undefined;
        this.applicationFormResourceState = undefined;
        this.questionPoolResourceState = undefined;
        this.questionSetResourceState = undefined;
        this.questionSetState = undefined;
        this.locked = undefined;
        this.pendingProcesses = undefined;
        this.isOwner = undefined;
        this.hasPermissionToEdit = undefined;
        this.isActiveQuestionSetPendingProcess = undefined;
        this.enableUserApplicationGuard = undefined;
        this.applicationId = undefined;
        this.addressValidationStatuses = undefined;
        this.addressIdsToBeValidated = undefined;
        this.isValidationTriggerOverridden = undefined;
        this.useExternal = false;
        this.canSave = true;
        this.areResourcesReady = true;
        this.isDisabled = false;
        this.caDisabled = false;
        this.isReady = true;
        this.isLocked = false;
        this.isFirstRun = true;
        this.classes = '';
        this.isActiveQuestionSetBeingValidated = undefined;
    }
    resetState() {
        this.isReady = true;
        this.isFirstRun = true;
    }
    connectedCallback() {
        this.resetState();
        this.handleSubscriptions();
    }
    componentWillLoad() {
        this.useExternal = !!customElements.get('bp-sdk-external-next-button-container');
        customElements
            .whenDefined('bp-sdk-external-next-button-container')
            .then(() => {
            this.useExternal = true;
        });
        this.handleSubscriptions();
        this.onShouldDisableIfIncompleteChange(this.shouldDisableIfIncomplete);
        this.onAnswersResourcesStateChange(this.answersResourceState);
        this.onApplicationFormResourceStateChange(this.applicationFormResourceState);
        this.onHasPermissionToEdit(this.hasPermissionToEdit);
        this.onEnableUserApplicationGuard(this.enableUserApplicationGuard);
        this.onIsFirstRun(this.isFirstRun);
        this.onIsOwner(this.isOwner);
        this.onIsReady(this.isReady);
        this.onLocked(this.locked);
        this.onQuestionPoolResourcesStateChange(this.questionPoolResourceState);
        this.onQuestionSetResourcesStateChange(this.questionSetResourceState);
        this.onQueueChange(this.queue);
        this.onIsActiveQuestionSetPendingProcess(this.isActiveQuestionSetPendingProcess);
        this.onAddressValidationStatusChange(this.addressValidationStatuses);
    }
    onEnableUserApplicationGuard(enableUserApplicationGuard) {
        this.updateIsLocked(this.locked, this.isOwner, this.hasPermissionToEdit, enableUserApplicationGuard);
    }
    onHasPermissionToEdit(hasPermission) {
        this.updateIsLocked(this.locked, this.isOwner, hasPermission, this.enableUserApplicationGuard);
    }
    onIsOwner(isOwner) {
        this.updateIsLocked(this.locked, isOwner, this.hasPermissionToEdit, this.enableUserApplicationGuard);
    }
    onLocked(locked) {
        this.updateIsLocked(locked, this.isOwner, this.hasPermissionToEdit, this.enableUserApplicationGuard);
    }
    updateIsLocked(locked, isOwner, hasPermission, enableUserApplicationGuard) {
        this.isLocked =
            locked || (enableUserApplicationGuard && (!isOwner || !hasPermission));
    }
    onIsFirstRun(isFirstRun) {
        this.updateAreResourcesReady(isFirstRun, this.isReady);
    }
    onIsReady(isReady) {
        this.updateAreResourcesReady(this.isFirstRun, isReady);
    }
    updateAreResourcesReady(isFirstRun, isReady) {
        this.areResourcesReady = isFirstRun || isReady;
    }
    onAnswersResourcesStateChange(state) {
        this.isReady = this.calculateResourcesReady({
            answers: state,
            applicationForm: this.applicationFormResourceState,
            questionPools: this.questionPoolResourceState,
            questionSets: this.questionSetResourceState,
        });
    }
    onApplicationFormResourceStateChange(state) {
        if (this.isFirstRun && state === index_esm$1.R.Loaded) {
            this.isFirstRun = false;
        }
        this.isReady = this.calculateResourcesReady({
            answers: this.answersResourceState,
            applicationForm: state,
            questionPools: this.questionPoolResourceState,
            questionSets: this.questionSetResourceState,
        });
    }
    onQuestionPoolResourcesStateChange(state) {
        this.isReady = this.calculateResourcesReady({
            answers: this.answersResourceState,
            applicationForm: this.applicationFormResourceState,
            questionPools: state,
            questionSets: this.questionSetResourceState,
        });
    }
    onQuestionSetResourcesStateChange(state) {
        this.isReady = this.calculateResourcesReady({
            answers: this.answersResourceState,
            applicationForm: this.applicationFormResourceState,
            questionPools: this.questionPoolResourceState,
            questionSets: state,
        });
    }
    calculateResourcesReady(resources = {}) {
        if (!resources ||
            resources.applicationForm !== index_esm$1.R.Loaded ||
            resources.answers === index_esm$1.R.Loading ||
            resources.questionPools === index_esm$1.R.Loading ||
            resources.questionSets === index_esm$1.R.Loading) {
            return false;
        }
        return true;
    }
    onShouldDisableIfIncompleteChange(shouldDisableIfIncomplete) {
        this.updateOnDisableIfIncomplete(shouldDisableIfIncomplete, this.activeQuestionSet);
    }
    onActiveQuestionSetChange(activeQuestionSet) {
        this.updateOnDisableIfIncomplete(this.shouldDisableIfIncomplete, activeQuestionSet);
        this.disableIfCAQuestionSet(this.shouldDisableIfIncomplete, activeQuestionSet);
    }
    onQueueChange(queue) {
        this.canSave = !(queue === null || queue === void 0 ? void 0 : queue.length) && !this.isActiveQuestionSetPendingProcess;
    }
    onAddressValidationStatusChange(statuses) {
        if (statuses &&
            this.activeQuestionSet &&
            Object.values(statuses).some((status) => status === index_esm$1.Se.PROCEED_NEXT ||
                status === index_esm$1.Se.VALIDATION_ERROR) &&
            !this.isValidationTriggerOverridden) {
            const filteredIds = Object.keys(statuses).filter((id) => this.activeQuestionSet.answers
                .map((answer) => answer.answer_pool_id)
                .includes(id) ||
                this.activeQuestionSet.question_set.question_groups
                    .map((group) => group.id)
                    .includes(id));
            if (filteredIds.every((id) => statuses[id] === index_esm$1.Se.PROCEED_NEXT ||
                statuses[id] === index_esm$1.Se.COMPLETED ||
                statuses[id] === index_esm$1.Se.VALIDATION_ERROR)) {
                filteredIds
                    .filter((id) => statuses[id] === index_esm$1.Se.PROCEED_NEXT)
                    .forEach((addressValidationId) => this.setAddressValidationStatus({
                    validationStatus: index_esm$1.Se.COMPLETED,
                    addressValidationId,
                }));
                filteredIds
                    .filter((id) => statuses[id] === index_esm$1.Se.VALIDATION_ERROR)
                    .forEach((addressValidationId) => this.setAddressValidationStatus({
                    validationStatus: index_esm$1.Se.REQUIRED,
                    addressValidationId,
                }));
                this.saveQuestionSetAndProceed();
            }
        }
    }
    onIsActiveQuestionSetPendingProcess(isQSPending) {
        this.canSave = !isQSPending;
    }
    disableIfCAQuestionSet(shouldDisableIfIncomplete, activeQuestionSet) {
        let shouldDisable = false;
        if (shouldDisableIfIncomplete && activeQuestionSet) {
            shouldDisable =
                index_esm$2.Ke(activeQuestionSet);
        }
        if (this.caDisabled !== shouldDisable) {
            this.caDisabled = shouldDisable;
        }
    }
    updateOnDisableIfIncomplete(shouldDisableIfIncomplete, activeQuestionSet) {
        this.isDisabled =
            shouldDisableIfIncomplete &&
                this.isDisabledStatusByActiveQuestionSet(activeQuestionSet);
    }
    render() {
        return [
            this.useExternal && this.renderExternal(),
            !this.useExternal && this.renderInternal(),
        ];
    }
    renderExternal() {
        return (index.h("bp-sdk-external-next-button-container", { activeQuestionSet: this.activeQuestionSet, answersResourceState: this.answersResourceState, applicationFormResourceState: this.applicationFormResourceState, applicationId: this.applicationId, buttonDisabled: this.buttonDisabled, config: this.config, enableUserApplicationGuard: this.enableUserApplicationGuard, hasPermissionToEdit: this.hasPermissionToEdit, isOwner: this.isOwner, locked: this.locked, pendingProcesses: this.pendingProcesses, questionPoolResourceState: this.questionPoolResourceState, questionSetResourceState: this.questionSetResourceState, questionSetState: this.questionSetState, queue: this.queue, shouldDisableIfIncomplete: this.shouldDisableIfIncomplete, isActiveQuestionSetPendingProcess: this.isActiveQuestionSetPendingProcess }));
    }
    handleNavigateNext() {
        if (this.isAddressValidationRequired()) {
            this.handleValidation();
        }
        else {
            this.saveQuestionSetAndProceed();
        }
    }
    renderInternal() {
        return (index.h("bp-sdk-next-button", { buttonDisabled: !this.isNextButtonClickable() ||
                this.isActiveQuestionSetBeingValidated, cssClasses: !this.areResourcesReady || !this.canSave ? 'loading' : 'not-loading' }));
    }
    parentQuestionSetCanSave() {
        const parentQuestionSet = this.questionSetState.parentQuestionSets.find((pqs) => pqs.id === this.activeQuestionSet.parentQuestionSetId);
        if (parentQuestionSet) {
            return this.questionSetState.list
                .filter((qs) => qs.parentQuestionSetId === parentQuestionSet.id)
                .every((qs) => index_esm$2.R(qs.answers));
        }
    }
    saveQuestionSetAndProceed() {
        if (this.shouldCallSaveQuestionSet()) {
            const invalidAnswers = this.activeQuestionSet.answers.filter((answer) => !index_esm$2.je(answer));
            if (!invalidAnswers.length && this.isNextButtonClickable()) {
                this.removeActiveAnswerId();
                if (!this.activeQuestionSet.parentQuestionSetId) {
                    this.callSaveQuestionSet();
                }
                else {
                    if (this.parentQuestionSetCanSave()) {
                        this.callSaveQuestionSet(true);
                    }
                    else {
                        this.handleStep();
                    }
                }
            }
            else {
                invalidAnswers.forEach((answer) => {
                    this.updateErrors({ answer, errors: index_esm$2.te(answer) });
                });
                index_esm$2._t();
            }
        }
        else if (this.isNextButtonClickable()) {
            this.switchActiveQuestionSet('forward');
        }
    }
    callSaveQuestionSet(useParent = false) {
        this.saveQuestionSet({
            config: this.config,
            appFormQuestionSet: this.activeQuestionSet,
            navigationDirection: 'forward',
            parentQuestionSet: useParent
                ? this.questionSetState.parentQuestionSets.find((pqs) => pqs.id === this.activeQuestionSet.parentQuestionSetId)
                : undefined,
        });
        this.bpAnalyticsTrack.emit({
            eventName: index_esm$1.Re.QuestionSetSaved,
            eventPayload: {
                applicationFormQuestionSetId: this.activeQuestionSet.id,
                questionSetId: this.activeQuestionSet.question_set_id,
                questionSetCode: this.activeQuestionSet.question_set.code,
            },
        });
    }
    handleStep() {
        const parent = index_esm$2.Fn(this.activeQuestionSet, this.questionSetState);
        const newValue = index_esm$2.Un(this.activeQuestionSet);
        if (parent.value !== newValue) {
            this.pushAnswer(index_esm$2.L(parent, newValue));
        }
        this.getEligibilityResults({
            appId: this.applicationId,
            config: this.config,
        });
        this.switchActiveQuestionSet('forward');
    }
    isDisabledStatusByActiveQuestionSet(activeQuestionSet) {
        var _a;
        // If missing active question set then default to disable.
        if (!activeQuestionSet) {
            return true;
        }
        // If missing answers then default to enabled
        if (!activeQuestionSet.answers || ((_a = activeQuestionSet.answers) === null || _a === void 0 ? void 0 : _a.length) === 0) {
            return false;
        }
        // Else state determined by if all required answers are completed
        return activeQuestionSet.answers.some((answer) => !answer.is_completed && answer.required);
    }
    shouldCallSaveQuestionSet() {
        if (this.isLocked) {
            return false;
        }
        return (!this.activeQuestionSet.is_completed ||
            !!this.activeQuestionSet.parentQuestionSetId);
    }
    isNextButtonClickable() {
        return (!this.buttonDisabled &&
            this.areResourcesReady &&
            this.canSave &&
            !this.isDisabled &&
            !this.caDisabled);
    }
    isAddressValidationRequired() {
        return Object.keys(this.addressValidationStatuses).some((id) => this.activeQuestionSet.answers
            .map((answer) => answer.answer_pool_id)
            .includes(id) ||
            this.activeQuestionSet.question_set.question_groups
                .map((group) => group.id)
                .includes(id));
    }
    handleValidation() {
        if (this.isValidationTriggerOverridden) {
            this.setAddressValidationTriggerOverridden(false);
        }
        const presentRequiredIds = this.addressIdsToBeValidated.filter((id) => this.activeQuestionSet.answers
            .map((answer) => answer.answer_pool_id)
            .includes(id) ||
            this.activeQuestionSet.question_set.question_groups
                .map((group) => group.id)
                .includes(id));
        if (!!presentRequiredIds.length) {
            presentRequiredIds.forEach((addressValidationId) => {
                this.setAddressValidationStatus({
                    validationStatus: index_esm$1.Se.READY_TO_VALIDATE,
                    addressValidationId,
                });
            });
        }
        else {
            const filteredIds = Object.keys(this.addressValidationStatuses).filter((id) => this.activeQuestionSet.answers
                .map((answer) => answer.answer_pool_id)
                .includes(id) ||
                this.activeQuestionSet.question_set.question_groups
                    .map((group) => group.id)
                    .includes(id));
            if (filteredIds.every((id) => this.addressValidationStatuses[id] === index_esm$1.Se.COMPLETED ||
                this.addressValidationStatuses[id] ===
                    index_esm$1.Se.VALIDATION_ERROR)) {
                this.saveQuestionSetAndProceed();
            }
        }
    }
    static get watchers() { return {
        "enableUserApplicationGuard": ["onEnableUserApplicationGuard"],
        "hasPermissionToEdit": ["onHasPermissionToEdit"],
        "isOwner": ["onIsOwner"],
        "locked": ["onLocked"],
        "isFirstRun": ["onIsFirstRun"],
        "isReady": ["onIsReady"],
        "answersResourceState": ["onAnswersResourcesStateChange"],
        "applicationFormResourceState": ["onApplicationFormResourceStateChange"],
        "questionPoolResourceState": ["onQuestionPoolResourcesStateChange"],
        "questionSetResourceState": ["onQuestionSetResourcesStateChange"],
        "shouldDisableIfIncomplete": ["onShouldDisableIfIncompleteChange"],
        "activeQuestionSet": ["onActiveQuestionSetChange"],
        "queue": ["onQueueChange"],
        "addressValidationStatuses": ["onAddressValidationStatusChange"],
        "isActiveQuestionSetPendingProcess": ["onIsActiveQuestionSetPendingProcess"]
    }; }
};
BpSdkNextButtonContainer.style = BpSdkNextButtonContainerStyle0;

exports.bp_sdk_next_button_container = BpSdkNextButtonContainer;
