import { IChoice, IDynamicInputComponent, QuestionFieldType, QuestionReferenceType, TAnswerDisplayType } from '@boldpenguin/sdk-types';
import { EventEmitter } from '../../stencil-public-runtime';
export declare class BpSdkDynamicInput implements IDynamicInputComponent {
    /**
     * Answer id
     */
    answerId: string;
    /**
     * Answer pool id
     */
    answerPoolId: string;
    /**
     * Application form id
     */
    applicationFormId: string;
    /**
     * Choice Id, should only be used on autocomplete, business-classification-selector, select, and radio inputs
     */
    choiceId: string | null;
    /**
     * Choice ids, should only be used on select-input or product-list
     */
    choiceIds: string[];
    /**
     * Choices, used in any component that uses choiceId or ChoiceIds
     */
    choices: IChoice[];
    /**
     * Question code
     */
    code: string;
    /**
     * Display type
     */
    displayType: TAnswerDisplayType;
    /**
     * Error messages
     */
    errors: string[];
    /**
     * Error text
     */
    errorText: string | null;
    /**
     * Field type
     */
    fieldType: QuestionFieldType;
    /**
     * Help text
     */
    helpText: string;
    /**
     * Input component type
     */
    inputComponentType: QuestionFieldType;
    /**
     * Is autocomplete
     */
    isAutocomplete: boolean;
    /**
     * Is completed
     */
    isCompleted: boolean;
    /**
     * Is Coverage Type
     */
    isCoverageType: boolean;
    /**
     * Is hidden
     */
    isHidden: boolean;
    /**
     * Is labelled field
     */
    isLabelledField: boolean;
    /**
     * Is multiple
     */
    isMultiple: boolean;
    /**
     * When true, the field will be readonly. It will display only the label and value as plain text.
     */
    isReadOnly: boolean;
    /**
     * Is required
     */
    isRequired: boolean;
    /**
     * Is sensitive
     */
    isSensitive: boolean;
    /**
     * Is trackable
     */
    isTrackable: boolean;
    /**
     * Label to use
     */
    label: string;
    /**
     * Label id
     */
    labelId: string;
    /**
     *  Labels that describe group
     */
    labelledBy: string;
    /**
     * Max date range
     */
    maxDate: string | Date | null;
    /**
     * Maximum length or value, depending on field type
     */
    maxValue: number | null;
    /**
     * Minimum date range
     */
    minDate: string | Date | null;
    /**
     * Minimum value or length depending on field type
     */
    minValue: number | null;
    /**
     * Question order
     */
    order: number;
    /**
     * Regex pattern answer is expected to conform to
     */
    pattern: string | null;
    /**
     * Placeholder text
     */
    placeholder: string | null;
    /**
     * Is pristine
     */
    pristine: boolean;
    /**
     * Question Id
     */
    questionId: string;
    /**
     * Question Group Id
     */
    questionGroupId: string;
    /**
     * Question Set id
     */
    questionSetId: string;
    /**
     * Question reference type
     */
    referenceType: QuestionReferenceType | null;
    /**
     * Show business class selector
     */
    showBusinessClassificationSelector: boolean;
    /**
     * Disables Google Autocomplete for address groups when true
     */
    disableAddressAutocomplete: boolean;
    /**
     * Should show input
     */
    shouldShowInput: boolean;
    /**
     * Is touched
     */
    touched: boolean;
    /**
     * Question text
     */
    text: string | null;
    /**
     * Use select alias
     */
    useSelectAlias: boolean;
    /**
     * Question value
     */
    value: string | null;
    /**
     * Specifically how the question was answered (ie. "ACORD 126")
     */
    answeredBySource: string | null;
    id: string;
    useExternal: boolean;
    /**
     * Notification component could not be classified for a answer
     */
    componentMissing: EventEmitter<string>;
    resetState(): void;
    connectedCallback(): void;
    componentWillLoad(): void;
    render(): any[];
    renderExternal(): any;
    renderInternal(): any;
    renderContent(): any;
    renderTextSwitch(): any;
    renderGooglePlacesAutocomplete(): any;
    renderDriverLicense(): any;
    renderIncrement(): any;
    renderText(): any;
    renderMaskedInput(): any;
    renderDate(): any;
    renderMonthYear(): any;
    renderDropdownSwitch(): any;
    renderDropdownFallback(): any;
    renderSelect(): any;
    renderBcsSwitch(): any;
    renderAutocomplete(): any;
    renderBcs(): any;
    renderDialog(): any;
    renderCheckbox(): any;
    renderRadio(): any;
    renderCoverageSelector(): any;
    renderQuestionPoolItemContainer(): any;
    renderDeletePool(): any;
    renderVehicleAutocomplete(): any;
}
